// Copyright (C) Geom Software e.U, Bernhard Kornberger, Graz/Austria
//
// This file is part of the WOF software. WOF is commercial software.
// Users holding a license may use this file in accordance with the
// License Agreement.
//
// This software is provided AS IS with NO WARRANTY OF ANY KIND,
// INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE.
//
// Please contact the author if any conditions of this licensing are
// not clear to you.
//
// Author: Bernhard Kornberger, bkorn (at) geom.at
// http://www.geom.at

#include <stdlib.h>
#include <string>
#include <iostream>
#include <iomanip>
#include "wof_api.h"

using namespace std;

// Operation examples
int operationReconstruct(); // Reconstruct a mesh from a point cloud
int operationRemesh(); // Remesh a triangle mesh
int operationToCloud(); // Mesh-to-Cloud

// Version and license-related example functions
bool greeting();  // Version and license information
int activate_user_only(); // Activate the license for the current user
int activate_system_wide(); // Activate the license system-wide
int deactivate(); // Deactivate the license (do not use frequently!)



// *** main() ***
int main(int,char**)
{
	// It makes sense to use try/catch blocks because unrecognized
	// file formats and parameters may trigger an exception
	try
	{
		// * 1 *   Inform about version and license
		bool bOK=greeting();
		if(!bOK) return 80;

		// * 2 *   Activate a license (uncomment license operations you want to test)
		//return activate_user_only();
		//return activate_system_wide();
		//return deactivate();

		// * 3 *   Operation (uncomment the operation you want to test)
		return operationReconstruct();
		//return operationRemesh();
		//return operationToCloud();
	}
	catch(...)
	{
		cout<<"Uh, failed"<<endl;
	}

}

/////////////////////////////////////////////////
// Operation examples
/////////////////////////////////////////////////


// Reconstruct a triangle mesh from a point cloud
int operationReconstruct()
{
	// * 1 *   Read the input point cloud
	std::string inFilename("../data/bunny_cloud_1M.ply"); // You can use *.ply, *.bin, (*.asc,*.txt,*.xyz)
	vector<GEOM_WOF::Point3> vPoints;
	GEOM_WOF::TimerC timerRead;
	bool bReadOK=GEOM_WOF::readPoints_auto(inFilename,vPoints);
	timerRead.stop();
	cout<<"Input points from ["<<inFilename<<"]: "<<vPoints.size()<<endl<<endl;

	if(!bReadOK || vPoints.empty())
	{
		cout<<"No points, stop"<<endl;
		return 1;
	}

	// * 2 *   Reconstruct
	GEOM_WOF::TimerC timerReco;
	bool bAutomaticDensity(true); // Choose true for automatic estimation or false to specify an absolute value
	bool bAllowSmoothing(true); // Choose true or false
	std::shared_ptr<GEOM_WOF::WofMesh> pWofMesh;
	if(bAutomaticDensity)
	{
		double spacingFactor(4); // Default is 2.0
		pWofMesh=GEOM_WOF::reconstruct_auto(vPoints,bAllowSmoothing,spacingFactor);
	}
	else
	{
		double absoluteSpacing(0.003); // Should approximately match with the point cloud
		pWofMesh=GEOM_WOF::reconstruct_abs(vPoints,bAllowSmoothing,absoluteSpacing);
	}
	timerReco.stop();


	// * 3 *   Output
	bool bASCII(true); // Use ASCII format if both, binary and ASCII are available
	std::string outFilename("result_bunny.ply"); // Use *.ply, *.stl, *.list (STL is always ASCII, not binary!)
	GEOM_WOF::TimerC timerWrite;
	bool bWriteOK=GEOM_WOF::writeMesh_auto(outFilename,pWofMesh,bASCII);
	if(!bWriteOK)
	{
		cout<<"File "<<outFilename<<" not written!"<<endl;
	}
	timerWrite.stop();

	pWofMesh->printStatistics(outFilename);

	// * 4 *   Show the time consumption
	cout<<"\nTime:"<<endl;
	cout<<"  I/O Read: "<<timerRead.get()<<" s"<<endl;
	cout<<"  Meshing:  "<<timerReco.get()<<" s"<<endl;
	cout<<"  I/O Write:"<<timerWrite.get()<<" s"<<endl;
	cout<<endl;

	if(bWriteOK) return 0; // Success
		else return 1;
}


// Create a point cloud from a mesh and reconstruct the mesh from the
// point cloud. This process is also known als 'melting': Small gaps
// are closed, parts inside a solid boundary are removed and self-
// intersections are removed. Melting does not create a better
// visual appearance but the output is better suited for further
// technical processing (simulations, ...)
int operationRemesh()
{
	// * 1 *   Read the triangle corners
	std::string inFile("../data/Music_Art_org.ply");
	GEOM_WOF::TimerC timerRead;
	std::vector<GEOM_WOF::Point3> vInCorners;
	switch(GEOM_WOF::getFileType(inFile))
	{
		case GEOM_WOF::FT_STL:
			GEOM_WOF::readSTL_ASCII(inFile.c_str(),vInCorners);
			break;
		case GEOM_WOF::FT_PLY:
			GEOM_WOF::readPly(inFile.c_str(),false,vInCorners);
			break;
		default:
			cerr<<"Unsupported input file format"<<endl;
			return 65; // data format error
	}
	cout<<"operationRemesh(): "<<endl;
	cout<<"  "<<inFile<<": "<<vInCorners.size()/3<<" triangles"<<endl;
	timerRead.stop();

	if(vInCorners.empty())
	{
		cout<<"No input points"<<endl;
		return 65; // data format error
	}

	// Melt the mesh (choose spacingAbs carefully: Time- and memory!)
	double spacingAbs=0.2; // Approximate edge length (use at most half the object's thickness)
	double featureAngle=15.0; // Threshold, edges with larger dihedral angles are feature edges (it is tried to keep them)
	GEOM_WOF::TimerC timerMelt;
		std::shared_ptr<GEOM_WOF::WofMesh> pWofMesh=GEOM_WOF::melt(vInCorners,spacingAbs,featureAngle);
	timerMelt.stop();



	// Write the output file
	std::string outFilename("result_Music_Art_remeshed.ply");
	pWofMesh->printStatistics(outFilename);
	bool bUseASCII(true);
	GEOM_WOF::TimerC timerWrite;
	bool bWriteOK=writeMesh_auto(outFilename,pWofMesh,bUseASCII);
	if(!bWriteOK)
	{
		cout<<"Writing "<<outFilename<<" failed"<<endl;
	}
	timerWrite.stop();

	// Show timing
	cout<<"\nTime:"<<endl;
	cout<<"  I/O Read: "<<timerRead.get()<<" s"<<endl;
	cout<<"  Remeshing:"<<timerMelt.get()<<" s"<<endl;
	cout<<"  I/O Write:"<<timerWrite.get()<<" s"<<endl;
	cout<<endl<<endl;
	cout<<"DONE!"<<endl;
	if(bWriteOK) return 0; // Success
		else return 1;
}

int operationToCloud()
{
	std::string inFile("../data/Music_Art_org.ply");

	// * 1 *   Read the triangle corners
	GEOM_WOF::TimerC timerRead;
	std::vector<GEOM_WOF::Point3> vInCorners;
	switch(GEOM_WOF::getFileType(inFile))
	{
		case GEOM_WOF::FT_STL:
			GEOM_WOF::readSTL_ASCII(inFile.c_str(),vInCorners);
			break;
		case GEOM_WOF::FT_PLY:
			GEOM_WOF::readPly(inFile.c_str(),false,vInCorners);
		break;

		default:
			cerr<<"Unsupported input file format"<<endl;
			return 65; // data format error
	}
	cout<<"operationToCloud:"<<endl;
	cout<<"  "<<inFile<<": "<<vInCorners.size()/3<<" triangles"<<endl;
	timerRead.stop();

	if(vInCorners.empty())
	{
		cout<<"No input points"<<endl;
		return 65; // data format error
	}

	double spacingAbs(0.3); // Approximate edge length
	double featureThreshold(15.0);

	// Melt the mesh
	GEOM_WOF::TimerC timerToCloud;
		vector<GEOM_WOF::Point3> vPointsOut;
		toCloud(vInCorners,spacingAbs,featureThreshold,vPointsOut);
	timerToCloud.stop();

	std::string outFilename("result_Music_Art_cloud.xyz"); // You can also use *.ply or *.bin
	cout<<"  "<<outFilename<<": "<<vPointsOut.size()<<" points"<<endl;

	// Write the output file
	bool bUseASCII(true);
	GEOM_WOF::TimerC timerWrite;
	bool bWriteOK=writePoints_auto(outFilename,vPointsOut,bUseASCII);
	if(!bWriteOK)
	{
		cout<<"Writing "<<outFilename<<" failed"<<endl;
	}
	timerWrite.stop();

	// Show timing
	cout<<"\nTime:"<<endl;
	cout<<"  I/O Read: "<<timerRead.get()<<" s"<<endl;
	cout<<"  toCloud:  "<<timerToCloud.get()<<" s"<<endl;
	cout<<"  I/O Write:"<<timerWrite.get()<<" s"<<endl;
	cout<<endl<<endl;
	cout<<"DONE!"<<endl;

	if(bWriteOK) return 0; // Success
		else return 1;
}




/////////////////////////////////////////////////
// Version- and license-related examples
/////////////////////////////////////////////////

bool greeting()
{
	// Version info
	int maj,min;
	GEOM_WOF::getVersion(maj,min);
	cout<<"WOF, version "<<maj<<"."<<setw(2)<<setfill('0')<<min<<" ";
	if(GEOM_WOF::isRelease()) cout<<", Buildtype: Release";
		else cout<<", Buildtype: Debug";

	// Which license do we have?
	switch(GEOM_WOF::getLicenseState())
	{
		case WOFLIC_ACTIVATED:
			cout<<", License: active"<<endl;
			return true;
		case WOFLIC_TRIAL:
			cout<<endl; // Trial and Days remaining are printed by limeMain()
			return true;
		case WOFLIC_GRACE_OK:
			cout<<", License: No internet connection (grace period)"<<endl;
			return true;
		case WOFLIC_GRACE_EXPIRED:
			cout<<"License: No internet connection, grace period expired, establish internet access to enable re-verification"<<endl;
			return false;
		case WOFLIC_INVALID:
		default:
			cout<<"License: No"<<endl;
			return false;
	}
}


// System-wide activation requires admin priviledges. Note: Activation
// shall only be made once. Then the software is activated and another
// call to this method will fail. You can check the licensing state
// with GEOM_WOF::getLicenseState().
int activate_system_wide()
{
	const char* yourKey("ABCD EFGH ...");
	if(GEOM_WOF::activateWof(yourKey,true))
	{
		return 0; // Success
	}
	else
	{
		return 80; // License error
	}
}

// Activation for the current user. Note: Activation shall only be made
// once. Then the software is activated and another call to this method
// will fail. You can check the licensing state with
// GEOM_WOF::getLicenseState().
int activate_user_only()
{
	const char* yourKey("ABCD EFGH ...");
	if(GEOM_WOF::activateWof(yourKey,false))
	{
		return 0; // Success
	}
	else
	{
		return 80; // License error
	}
}

// Deactivate so that the key can be used for another machine. Do this
// only when an old computer is replaced. The number of deactivations
// is limited.
int deactivate()
{
	if(GEOM_WOF::deactivateWof())
	{
		return 0; // Success
	}
	else
	{
		return 80; // License error
	}
}
