// Copyright (C) Geom Software e.U, Bernhard Kornberger, Graz/Austria
//
// This file is part of the WOF software. WOF is commercial software.
// Users holding a license may use this file in accordance with the
// License Agreement.
//
// This software is provided AS IS with NO WARRANTY OF ANY KIND,
// INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE.
//
// Please contact the author if any conditions of this licensing are
// not clear to you.
//
// Author: Bernhard Kornberger, bkorn (at) geom.at
// http://www.geom.at

#pragma once

#include <vector>
#include <memory>
#include "Point3.h"

namespace GEOM_WOF {

struct Dat; // FWD
class WofMesh; // FWD
typedef std::shared_ptr<WofMesh> WMeshPtr;

/** \brief 3D Mesh class
 *
 * The WofMesh is a 3D triangle mesh.
 */
class CLASS_DECLSPEC WofMesh
{
public:
	/** \brief Constructor
	 */
	explicit WofMesh(Dat* pDat_);
	/** \brief Destructor
	 */
	~WofMesh();
	/** \brief Get Triangles
	 *
	 * @param [out] vTriangleCorners is used to return the triangles
	 * as 3 vertex pointers per triangle. The order of the corners
	 * per triangle is counterclockwise.
	 */
	void getTriangles(std::vector<Point3*>& vTriangleCorners) const;
	/** \brief Get Points
	 *
	 * @param [out] vPoints is used to return the vertex pointers
	 */
	void getPoints(std::vector<Point3*>& vPoints) const;
	/** \brief Write Ply (Binary)
	 *
	 * Writes a binary PLY file
	 *
	 * \param name [in] is the output filename.
	 */
	bool writePly_BIN(const std::string& name) const;
	/** \brief Write Ply (ASCII)
	 *
	 * Writes an ASCII PLY file
	 *
	 * \param name [in] is the output filename.
	 */
	bool writePly_ASCII(const std::string& name) const;
	/** \brief Write Geomview
	 *
	 * Writes a file for the Geomview viewer
	 *
	 * \param name [in] is the output filename.
	 */
	bool writeGeomview_ASCII(const std::string& name) const;
	/** \brief Write STL (ASCII)
	 *
	 * Writes an ASCII STL file
	 *
	 * \param name [in] is the output filename.
	 *
	 */
	bool writeStl_ASCII(const std::string& name) const;
	/** \brief Print Statistics
	 *
	 * Prints mesh statistics to stdout
	 *
	 * @param name serves as arbitrary identifier that is also printed to stdout
	 */
	void printStatistics(const std::string& name) const;
private:
	WofMesh(WofMesh&);
	Dat* pDat;
};



} // NAMESPACE


