// Copyright (C) Geom Software e.U, Bernhard Kornberger, Graz/Austria
//
// This file is part of the WOF software. WOF is commercial software.
// Users holding a license may use this file in accordance with the
// License Agreement.
//
// This software is provided AS IS with NO WARRANTY OF ANY KIND,
// INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE.
//
// Please contact the author if any conditions of this licensing are
// not clear to you.
//
// Author: Bernhard Kornberger, bkorn (at) geom.at
// http://www.geom.at

#pragma once

#include <vector>
#include "Point3.h"
#include "WofMesh.h"

namespace GEOM_WOF {

struct Dat; // FWD

/** \brief Geometry operations
 *
 * Operatios to reconstruct a surface, to sample-and-reconstruct a mesh and to 
 * create feature related quality point clouds.
 */

/** \defgroup geometryFun Geometry functions
 *  @{
 */

/** \brief Retriangulate a triangle mesh
*
* This function takes a triangular mesh and a distance. It
* pixels the input surface and reconstructs the mesh.
*
* @param [in] vInputCorners contains the input triangles (3 corners per triangle)
* @param [in] avgLength specifies the average distance to be used
* @param [in] featureThresh is an optional parameter. It specifies
* that edges above this value shall be treated as feature lines.
*
* @warning Choose @p avgLength with care. Large area meshes can
* cause an extreme number of elements.
*/
CLASS_DECLSPEC
std::shared_ptr<WofMesh> melt(	std::vector<Point3>& vInputCorners,
								double avgLength,
								double featureThresh=15.0
							);

/** \brief Reconstruct with an automatic spacing value
*
* This function takes a 3D point cloud and reconstructs a triangular
* mesh.
*
* @param [in] vPoints contains the input point cloud
* @param [in] sfactor influences the mesh density. The average
* spacing in the point cloud is automatically estimated and
* multiplied by sfactor. Use larger values to avoid holes and
* to create coarser meshes. By default sfactor=2.0.
* @param [in] bAllowSmoothing specifies if the point cloud shall be
* smoothed before reconstruction.
* @return the reconstructed mesh.
*/
CLASS_DECLSPEC
std::shared_ptr<WofMesh> reconstruct_auto(std::vector<Point3>& vPoints,bool bAllowSmoothing,double sfactor=2.0);

/** \brief Reconstruct with an absolute spacing value
*
* This function takes a 3D point cloud and reconstructs a triangular
* mesh.
*
* @param [in] vPoints contains the input point cloud
* @param [in] spacingAbs is an absolute spacing value. Large values
* @param [in] bAllowSmoothing specifies if the point cloud shall be
* smoothed before reconstruction.
* avoid holes and create coarser meshes.
* @return the reconstructed mesh.
*
* When the absolute spacing value is unknown then better use
* reconstruct_auto().
*/
CLASS_DECLSPEC
std::shared_ptr<WofMesh> reconstruct_abs(std::vector<Point3>& vPoints,bool bAllowSmoothing,double spacingAbs);


/** \brief Mesh-to-Cloud
*
* This function takes a 3D mesh and a distance. It transforms the mesh
* into a 3D point cloud
*
* @param [in] vCornersIn contains the input triangles (3 corners per triangle)
* @param [in] length specifies the approximate distance to be used
* @param [in] featureThresh is an optional parameter. Edges whose
* dihedral angle is beyond featureThresh are treated as feature edges
* @param [out] vCloudOut is used to return the point cloud
*/
CLASS_DECLSPEC
void toCloud(	std::vector<Point3>& vCornersIn,
				double length,
				double featureThresh,
				std::vector<Point3>& vCloudOut
				);

/** @}*/



} // NAMESPACE


